﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open System.ComponentModel.DataAnnotations
open Microsoft.Extensions.Caching.Memory
open System

// Define the Models module for Gas Balance
module GasBalanceModels =
    type GasBalanceItem = {
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
        [<JsonPropertyName("imbalance_buy_price")>]
        ImbalanceBuyPrice: float
        [<JsonPropertyName("imbalance_sell_price")>]
        ImbalanceSellPrice: float
    }

    type GasBalanceResponse = {
        [<JsonPropertyName("success")>]
        Success: bool
        [<JsonPropertyName("data")>]
        Data: GasBalanceItem list
    }

[<ApiController>]
[<Route("api/[controller]")>]
type GasBalanceController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    // Define the route to fetch gas balance data
    [<HttpGet("getgasbalance")>]
    member this.GetGasBalance([<Required;FromQuery>] start: string, [<Required;FromQuery>] ``end``: string) : Task<IActionResult> =
        async {
            // Generate a cache key using the start and end parameters
            let cacheKey = sprintf "gas_balance_%s_%s" start ``end``

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                // Construct the URL dynamically using the start and end parameters
                let url = sprintf "https://dashboard.elering.ee/api/gas-balance/price?start=%s&end=%s" start ``end``

                try
                    // Make the HTTP GET request
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask

                    if response.IsSuccessStatusCode then
                        // Read the response content as a string
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize the JSON string into GasBalanceResponse type
                        let gasBalanceResponse = JsonSerializer.Deserialize<GasBalanceModels.GasBalanceResponse>(json, options)

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, gasBalanceResponse, cacheOptions)

                        // Return the deserialized data as JSON
                        return this.Ok(gasBalanceResponse) :> IActionResult
                    else
                        // Return an error if the request was not successful
                        return this.StatusCode(int response.StatusCode, "Error fetching data from Elering API.") :> IActionResult
                with
                | ex -> 
                    // Handle errors and return an error response
                    return this.StatusCode(500, $"Internal server error: {ex.Message}") :> IActionResult
        } |> Async.StartAsTask
