﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open Microsoft.Extensions.Caching.Memory
open System

// Define the Models module
module GroupLatestPriceModels =
    type PriceItem = {
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
        [<JsonPropertyName("price")>]
        Price: float
    }

    type PriceResponse = {
        [<JsonPropertyName("success")>]
        Success: bool
        [<JsonPropertyName("data")>]
        Data: PriceItem list
    }

// Controller implementation
[<ApiController>]
[<Route("api/[controller]")>]
type GetGroupLatestPriceController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    // api/getgrouplatestprice/getgrouplatestprice/{group}
    [<HttpGet("getgrouplatestprice/{group}")>]
    member this.ShowPrice(group: string) : Task<IActionResult> =
        async {
            // Generate a cache key using the group parameter
            let cacheKey = sprintf "group_latest_price_%s" group

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                // Construct the URL dynamically using the group parameter
                let url = sprintf "https://dashboard.elering.ee/api/nps/price/%s/latest" group

                try
                    // Make the HTTP GET request
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask

                    if response.IsSuccessStatusCode then
                        // Read the response content as a string
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize the JSON string into PriceResponse type
                        let priceResponse = JsonSerializer.Deserialize<GroupLatestPriceModels.PriceResponse>(json, options)

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, priceResponse, cacheOptions)

                        // Directly return the deserialized data as JSON
                        return this.Ok(priceResponse) :> IActionResult
                    else
                        // Return an error if the request was not successful
                        return this.StatusCode(int response.StatusCode, "Error fetching data from Elering API.") :> IActionResult
                with
                | ex -> 
                    // Handle errors
                    return this.StatusCode(500, $"Internal server error: {ex.Message}") :> IActionResult
        } |> Async.StartAsTask
