﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open System.ComponentModel.DataAnnotations
open Microsoft.Extensions.Caching.Memory
open System

// Define the Models module
module PriceModels =
    type PriceItem = {
        [<JsonPropertyName("price")>]
        Price: float
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
    }

    type PriceData = {
        [<JsonPropertyName("ee")>]
        EE: PriceItem list
        [<JsonPropertyName("fi")>]
        FI: PriceItem list
        [<JsonPropertyName("lt")>]
        LT: PriceItem list
        [<JsonPropertyName("lv")>]
        LV: PriceItem list
    }

    type PriceResponse = {
        [<JsonPropertyName("data")>]
        Data: PriceData
        [<JsonPropertyName("success")>]
        Success: bool
    }


[<ApiController>]
[<Route("api/[controller]")>]
type PriceController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    // api/price/getprice/{start}/{end}

    [<HttpGet("getprice")>]
    member this.GetPrice([<Required;FromQuery>]start: string,[<Required;FromQuery>] ``end``: string) : Task<IActionResult> =
        async {
            let start = this.HttpContext.Request.Query.["start"].ToString()
            let end_ = this.HttpContext.Request.Query.["end"].ToString()

            // Generate a cache key using the start and end parameters
            let cacheKey = sprintf "price_%s_%s" start ``end``

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                let url = sprintf "https://dashboard.elering.ee/api/nps/price?start=%s&end=%s" start ``end``

                try
                    // Create an HTTP client and fetch data from the URL
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask

                    if response.IsSuccessStatusCode then
                        // Read the response content as a string
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize the JSON string into PriceResponse type
                        let priceResponse = JsonSerializer.Deserialize<PriceModels.PriceResponse>(json, options)

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, priceResponse, cacheOptions)

                        // Directly return the deserialized data as JSON
                        return this.Ok(priceResponse) :> IActionResult
                    else
                        // Return an error if the request was not successful
                        return this.StatusCode(int response.StatusCode, "Error fetching data from Elering API.") :> IActionResult
                with
                | ex ->
                    // Handle exceptions and return error response
                    return this.StatusCode(500, $"An error occurred: {ex.Message}") :> IActionResult
        } |> Async.StartAsTask
