﻿namespace EleringApi.Controllers
open Microsoft.AspNetCore.Mvc
open System.ComponentModel.DataAnnotations
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open Microsoft.Extensions.Caching.Memory
open System

// 1. Define the models for planned trade
module PlannedTradeModels =

    // Represents a single item in the "data" array
    type PlannedTradeItem = {
        [<JsonPropertyName("ee_fi")>]
        Eefi: float
        [<JsonPropertyName("ee_lv")>]
        Eelv: float
        [<JsonPropertyName("ee_ru")>]
        Eeru: float
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
    }

    // Represents the overall JSON response with "data" and "success"
    type PlannedTradeResponse = {
        [<JsonPropertyName("data")>]
        Data: PlannedTradeItem list
        [<JsonPropertyName("success")>]
        Success: bool
    }

// 2. Implement the controller
[<ApiController>]
[<Route("api/[controller]")>]
type PlannedTradeController(httpClient: HttpClient, memoryCache: IMemoryCache) =
    inherit ControllerBase()

    [<HttpGet("getplannedtrade")>]
    member this.GetPlannedTrade(
        [<Required; FromQuery>] start: string,
        [<Required; FromQuery>] ``end``: string
    ) : Task<IActionResult> =
        async {
            // 1. Build a unique cache key for the request
            let cacheKey = sprintf "plannedTrade:%s:%s" start ``end``

            // 2. Try to retrieve the cached response
            match memoryCache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Cache hit: return the cached data immediately
                return this.Ok(cachedData) :> IActionResult
            | false, _ ->
                // Cache miss: proceed with the external API call
                let url =
                    sprintf
                        "https://dashboard.elering.ee/api/transmission/cross-border-planned-trade?start=%s&end=%s"
                        start
                        ``end``

                try
                    // Make the HTTP GET request
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask

                    if response.IsSuccessStatusCode then
                        // Read the response content
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize the JSON into the PlannedTradeResponse type
                        let plannedTrade =
                            JsonSerializer.Deserialize<PlannedTradeModels.PlannedTradeResponse>(json, options)

                        // 3. Cache the result for 5 minutes
                        memoryCache.Set(
                            cacheKey,
                            plannedTrade,
                            MemoryCacheEntryOptions(AbsoluteExpirationRelativeToNow = TimeSpan.FromMinutes(7200.0))
                        )
                        |> ignore

                        // Return the newly fetched data
                        return this.Ok(plannedTrade) :> IActionResult
                    else
                        // Return an error if the request was not successful
                        return this.StatusCode(
                            int response.StatusCode,
                            "Error fetching data from the Elering API."
                        ) :> IActionResult
                with
                | ex ->
                    // Handle exceptions and return a 500 Internal Server Error
                    return this.StatusCode(
                        500,
                        $"Internal server error: {ex.Message}"
                    ) :> IActionResult
        } |> Async.StartAsTask