﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open Microsoft.Extensions.Caching.Memory
open System

/// Models for planned trade
module PlannedTradeLatestModels =
    /// Single data item in the "data" array
    type PlannedTradeLatestItem = {
        [<JsonPropertyName("ee_fi")>]
        Eefi: float
        [<JsonPropertyName("ee_lv")>]
        Eelv: float
        [<JsonPropertyName("ee_ru")>]
        Eeru: float
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
    }

    /// Overall response with "data" and "success"
    type PlannedTradeResponse = {
        [<JsonPropertyName("data")>]
        Data: PlannedTradeLatestItem list
        [<JsonPropertyName("success")>]
        Success: bool
    }

[<ApiController>]
[<Route("api/[controller]")>]
type PlannedTradeLatestController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    // GET /api/plannedtrade/getplannedtradelatest
    [<HttpGet("getplannedtradelatest")>]
    member this.GetPlannedTradeLatest() : Task<IActionResult> =
        async {
            // Cache key for the planned trade data
            let cacheKey = "planned_trade_latest"

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                // Endpoint for the latest planned trade data
                let url = "https://dashboard.elering.ee/api/transmission/cross-border-planned-trade/latest"

                try
                    // Make the GET request
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask

                    if response.IsSuccessStatusCode then
                        // Read the response body
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize into the planned trade model
                        let plannedTrade =
                            JsonSerializer.Deserialize<PlannedTradeLatestModels.PlannedTradeResponse>(json, options)

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, plannedTrade, cacheOptions)

                        // Return the data as JSON
                        return this.Ok(plannedTrade) :> IActionResult
                    else
                        // Return an error if the request failed
                        return this.StatusCode(
                            int response.StatusCode,
                            "Error fetching data from the Elering API."
                        ) :> IActionResult
                with
                | ex ->
                    // Handle exceptions
                    return this.StatusCode(
                        500,
                        $"Internal server error: {ex.Message}"
                    ) :> IActionResult
        } |> Async.StartAsTask
