﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open System.ComponentModel.DataAnnotations
open Microsoft.Extensions.Caching.Memory
open System

// 1. Define models for Transmission Border Capacity
module TransmissionBorderCapacityModels =

    /// Represents a single capacity item with 'in_atc', 'in_ntc', etc.
    type CapacityItem = {
        [<JsonPropertyName("in_atc")>]
        InAtc: float
        [<JsonPropertyName("in_ntc")>]
        InNtc: float
        [<JsonPropertyName("out_atc")>]
        OutAtc: float
        [<JsonPropertyName("out_ntc")>]
        OutNtc: float
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
    }

    /// Represents the 'data' object containing FI, LV, and RU lists
    type BorderData = {
        [<JsonPropertyName("FI")>]
        FI: CapacityItem list
        [<JsonPropertyName("LV")>]
        LV: CapacityItem list
        [<JsonPropertyName("RU")>]
        RU: CapacityItem list
    }

    /// The top-level JSON response with 'data' and 'success'
    type TransmissionBorderCapacityResponse = {
        [<JsonPropertyName("data")>]
        Data: BorderData
        [<JsonPropertyName("success")>]
        Success: bool
    }

[<ApiController>]
[<Route("api/[controller]")>]
type TransmissionBorderCapacityController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    /// Example usage:
    ///   GET /api/transmissionbordercapacity/getcapacity?start=2020-05-31T20%3A59%3A59.999Z&end=2020-06-30T20%3A59%3A59.999Z
    [<HttpGet("getcapacity")>]
    member this.GetCapacity([<Required; FromQuery>] start: string, [<Required; FromQuery>] ``end``: string) : Task<IActionResult> =
        async {
            // Generate a cache key based on the start and end dates
            let cacheKey = sprintf "capacity_%s_%s" start ``end``

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                // Construct the URL dynamically
                let url =
                    sprintf
                        "https://dashboard.elering.ee/api/transmission/cross-border-capacity?start=%s&end=%s"
                        start
                        ``end``

                try
                    // Make the HTTP GET request
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask
                    if response.IsSuccessStatusCode then
                        // Read the response content as a string
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize into TransmissionBorderCapacityResponse
                        let capacityResponse =
                            JsonSerializer.Deserialize<TransmissionBorderCapacityModels.TransmissionBorderCapacityResponse>(
                                json,
                                options
                            )

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, capacityResponse, cacheOptions)

                        // Return the deserialized data as JSON (HTTP 200 OK)
                        return this.Ok(capacityResponse) :> IActionResult
                    else
                        // Return an error if the request was not successful
                        return this.StatusCode(
                            int response.StatusCode,
                            "Error fetching data from Elering API."
                        ) :> IActionResult
                with
                | ex ->
                    // Handle exceptions and return a 500 Internal Server Error
                    return this.StatusCode(
                        500,
                        $"Internal server error: {ex.Message}"
                    ) :> IActionResult
        } |> Async.StartAsTask
