﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open System.ComponentModel.DataAnnotations
open Microsoft.Extensions.Caching.Memory
open System

// 1. Define the Models for Transmission Hourly
module TransmissionHourlyModels =
    /// Represents a single item in the "data" array of the JSON response
    type TransmissionHourlyItem = {
        [<JsonPropertyName("estlink_1")>]
        Estlink1: float
        [<JsonPropertyName("estlink_2")>]
        Estlink2: float
        [<JsonPropertyName("finland")>]
        Finland: float
        [<JsonPropertyName("latvia")>]
        Latvia: float
        [<JsonPropertyName("russia_narva")>]
        RussiaNarva: float
        [<JsonPropertyName("russia_pihkva")>]
        RussiaPihkva: float
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
    }

    /// Represents the overall JSON response with "data" and "success"
    type TransmissionHourlyResponse = {
        [<JsonPropertyName("data")>]
        Data: TransmissionHourlyItem list
        [<JsonPropertyName("success")>]
        Success: bool
    }

[<ApiController>]
[<Route("api/[controller]")>]
type TransmissionHourlyController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    // 3. Define an endpoint that returns the cached or fetched JSON
    [<HttpGet("gettransmissionhourly")>]
    member this.GetTransmissionHourly([<Required; FromQuery>] start: string, [<Required; FromQuery>] ``end``: string) : Task<IActionResult> =
        async {
            // Generate a cache key based on the start and end dates
            let cacheKey = sprintf "transmission_hourly_%s_%s" start ``end``

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                // Construct the URL dynamically using the query parameters
                let url = sprintf 
                            "https://dashboard.elering.ee/api/transmission/cross-border/hourly?start=%s&end=%s" 
                            start ``end``

                try
                    // Make the HTTP GET request
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask
                    if response.IsSuccessStatusCode then
                        // Read the response content as a string
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize the JSON into our TransmissionHourlyResponse model
                        let transmissionHourlyResponse =
                            JsonSerializer.Deserialize<TransmissionHourlyModels.TransmissionHourlyResponse>(json, options)

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, transmissionHourlyResponse, cacheOptions)

                        // Return the deserialized data as JSON
                        return this.Ok(transmissionHourlyResponse) :> IActionResult
                    else
                        // Return an error if the request was not successful
                        return this.StatusCode(
                            int response.StatusCode,
                            "Error fetching data from the Elering API."
                        ) :> IActionResult
                with
                | ex ->
                    // Handle any exceptions (network failures, parse errors, etc.)
                    return this.StatusCode(500, $"Internal server error: {ex.Message}") :> IActionResult
        } |> Async.StartAsTask
