﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open Microsoft.Extensions.Caching.Memory
open System

module TransmissionModels =
    /// Represents a single transmission item in the "data" array
    type TransmissionItem = {
        [<JsonPropertyName("estlink_1")>]
        Estlink1: float
        [<JsonPropertyName("estlink_2")>]
        Estlink2: float
        [<JsonPropertyName("finland")>]
        Finland: float
        [<JsonPropertyName("latvia")>]
        Latvia: float
        [<JsonPropertyName("russia_narva")>]
        RussiaNarva: float
        [<JsonPropertyName("russia_pihkva")>]
        RussiaPihkva: float
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
    }

    /// Represents the entire JSON response structure
    type TransmissionResponse = {
        [<JsonPropertyName("data")>]
        Data: TransmissionItem list
        [<JsonPropertyName("success")>]
        Success: bool
    }

[<ApiController>]
[<Route("api/[controller]")>]
type TransmissionLatestController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    [<HttpGet("transmissionlatest")>]
    member this.GetTransmission() : Task<IActionResult> =
        async {
            // Cache key for the latest transmission data
            let cacheKey = "transmission_latest"

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                // Elering API endpoint for cross-border latest transmission data
                let url = "https://dashboard.elering.ee/api/transmission/cross-border/latest"

                try
                    // Make the HTTP GET request to fetch the JSON
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask

                    if response.IsSuccessStatusCode then
                        // Read the response content as a string
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize the JSON response into our TransmissionResponse type
                        let transmissionResponse = JsonSerializer.Deserialize<TransmissionModels.TransmissionResponse>(json, options)

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, transmissionResponse, cacheOptions)

                        // Return the deserialized data as a JSON response
                        return this.Ok(transmissionResponse) :> IActionResult
                    else
                        // Return an appropriate status code and message if the API call failed
                        return this.StatusCode(int response.StatusCode, "Error fetching data from Elering API.") :> IActionResult
                with
                | ex ->
                    // Handle exceptions and return a 500 Internal Server Error
                    return this.StatusCode(500, $"Internal server error: {ex.Message}") :> IActionResult
        } |> Async.StartAsTask
