﻿namespace EleringApi.Controllers

open Microsoft.AspNetCore.Mvc
open System.Net.Http
open System.Text.Json
open System.Text.Json.Serialization
open System.Threading.Tasks
open System.ComponentModel.DataAnnotations
open Microsoft.Extensions.Caching.Memory
open System

// Define the Models module
module TurnoverModels =
    type TurnoverItem = {
        [<JsonPropertyName("in")>]
        In: float
        [<JsonPropertyName("out")>]
        Out: float
        [<JsonPropertyName("timestamp")>]
        Timestamp: int64
    }

    type TurnoverData = {
        [<JsonPropertyName("ee")>]
        EE: TurnoverItem list
        [<JsonPropertyName("fi")>]
        FI: TurnoverItem list
        [<JsonPropertyName("lt")>]
        LT: TurnoverItem list
        [<JsonPropertyName("lv")>]
        LV: TurnoverItem list
    }

    type TurnoverResponse = {
        [<JsonPropertyName("data")>]
        Data: TurnoverData
        [<JsonPropertyName("success")>]
        Success: bool
    }

[<ApiController>]
[<Route("api/[controller]")>]
type GetTurnoverController(httpClient: HttpClient, cache: IMemoryCache) =
    inherit ControllerBase()

    // api/getturnover/{start}/{end}
    [<HttpGet("getturnover")>]
    member this.ShowTurnover([<Required; FromQuery>] start: string,
        [<Required; FromQuery>] ``end``: string) : Task<IActionResult> =
        async {
            // Generate a cache key using the start and end parameters
            let cacheKey = sprintf "turnover_%s_%s" start ``end``

            match cache.TryGetValue(cacheKey) with
            | true, cachedData ->
                // Return cached data if it exists
                return this.Ok(cachedData) :> IActionResult
            | _ ->
                // Construct the URL dynamically using the start and end parameters
                let url = sprintf "https://dashboard.elering.ee/api/nps/turnover?start=%s&end=%s" start ``end``

                try
                    // Create an HTTP client and fetch data from the URL
                    let! response = httpClient.GetAsync(url) |> Async.AwaitTask

                    if response.IsSuccessStatusCode then
                        // Read the response content as a string
                        let! json = response.Content.ReadAsStringAsync() |> Async.AwaitTask
                        let options = JsonSerializerOptions(PropertyNameCaseInsensitive = true)

                        // Deserialize the JSON string into TurnoverResponse type
                        let priceResponse = JsonSerializer.Deserialize<TurnoverModels.TurnoverResponse>(json, options)

                        // Store the deserialized data in the cache
                        let cacheOptions = MemoryCacheEntryOptions().SetSlidingExpiration(TimeSpan.FromMinutes(7200.0))
                        cache.Set(cacheKey, priceResponse, cacheOptions)

                        // Directly return the deserialized data as JSON
                        return this.Ok(priceResponse) :> IActionResult
                    else
                        // Return an error if the request was not successful
                        return this.StatusCode(int response.StatusCode, "Error fetching data from Elering API.") :> IActionResult
                with
                | ex -> 
                    // Handle errors
                    return this.StatusCode(500, $"Internal server error: {ex.Message}") :> IActionResult
        } |> Async.StartAsTask
