import React, { useState, useEffect } from "react";
import {
  AreaChart,
  Area,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  ResponsiveContainer,
  BarChart,
  Bar,
} from "recharts";
import axios from "axios";

// Define types for the API responses
type RevenueItem = {
  timestamp: number; // Unix timestamp
  finland: number;
  latvia: number;
  estlink_1: number;
  estlink_2: number;
  russia_narva: number;
  russia_pihkva: number;
};

type WeeklyItem = {
  timestamp: number; // Unix timestamp
  ee_fi: number;
  ee_lv: number;
  ee_ru: number;
};

type RevenueData = {
  timestamp: string;
  finland: number;
  latvia: number;
  estlink_1: number;
  estlink_2: number;
  russia_narva: number;
  russia_pihkva: number;
};

type WeeklyData = {
  timestamp: string;
  ee_fi: number;
  ee_lv: number;
  ee_ru: number;
};

const Dashboard: React.FC = () => {
  const [revenueData, setRevenueData] = useState<RevenueData[]>([]);
  const [weeklyData, setWeeklyData] = useState<WeeklyData[]>([]);
  const [loading, setLoading] = useState<boolean>(false);

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        // Fetch Revenue Overview data (Transmission Latest)
        const revenueResponse = await axios.get(
          "https://localhost:7224/api/TransmissionLatest/transmissionlatest"
        );
        const revenue: RevenueItem[] = revenueResponse.data.data;

        if (revenue && revenue.length > 0) {
          // Prepare the revenue data for the area chart
          const revenueChartData: RevenueData[] = revenue.map((item) => ({
            timestamp: new Date(item.timestamp * 1000).toLocaleDateString(), // Format timestamp
            finland: item.finland,
            latvia: item.latvia,
            estlink_1: item.estlink_1,
            estlink_2: item.estlink_2,
            russia_narva: item.russia_narva,
            russia_pihkva: item.russia_pihkva,
          }));

          setRevenueData(revenueChartData);
        }

        // Fetch Weekly Activity data (Planned Trade Latest)
        const weeklyResponse = await axios.get(
          "https://localhost:7224/api/PlannedTradeLatest/getplannedtradelatest"
        );
        const weekly: WeeklyItem[] = weeklyResponse.data.data;

        if (weekly && weekly.length > 0) {
          // Prepare the weekly activity data for the bar chart
          const weeklyChartData: WeeklyData[] = weekly.map((item) => ({
            timestamp: new Date(item.timestamp * 1000).toLocaleDateString(),
            ee_fi: item.ee_fi,
            ee_lv: item.ee_lv,
            ee_ru: item.ee_ru,
          }));

          setWeeklyData(weeklyChartData);
        }
      } catch (error) {
        console.error("Error fetching data:", error);
      } finally {
        setLoading(false);
      }
    };

    fetchData();
  }, []);

  return (
    <div className="space-y-6">
      {/* Charts Grid */}
      <div className="grid grid-cols-1 lg:grid-cols-2 gap-6">
        {/* Show loading indicator while data is loading */}
        {loading ? (
          <div className="bg-white rounded-lg shadow p-6 text-center">
            <p>Loading...</p>
          </div>
        ) : (
          <>
            {/* Area Chart (Revenue Overview) */}
            <div className="bg-white rounded-lg shadow p-6">
              <h2 className="text-lg font-semibold text-gray-800 mb-4">
                Transmission Latest
              </h2>
              <div className="h-80">
                <ResponsiveContainer width="100%" height="100%">
                  <AreaChart data={revenueData}>
                    <CartesianGrid strokeDasharray="3 3" />
                    <XAxis dataKey="timestamp" />
                    <YAxis />
                    <Tooltip />
                    <Area
                      type="monotone"
                      dataKey="finland"
                      stroke="#4F46E5"
                      fill="#4F46E5"
                      fillOpacity={0.1}
                    />
                    <Area
                      type="monotone"
                      dataKey="latvia"
                      stroke="#00C49F"
                      fill="#00C49F"
                      fillOpacity={0.1}
                    />
                    <Area
                      type="monotone"
                      dataKey="estlink_1"
                      stroke="#FFBB28"
                      fill="#FFBB28"
                      fillOpacity={0.1}
                    />
                    <Area
                      type="monotone"
                      dataKey="estlink_2"
                      stroke="#FF8042"
                      fill="#FF8042"
                      fillOpacity={0.1}
                    />
                    <Area
                      type="monotone"
                      dataKey="russia_narva"
                      stroke="#FF63F4"
                      fill="#FF63F4"
                      fillOpacity={0.1}
                    />
                    <Area
                      type="monotone"
                      dataKey="russia_pihkva"
                      stroke="#FF6347"
                      fill="#FF6347"
                      fillOpacity={0.1}
                    />
                  </AreaChart>
                </ResponsiveContainer>
              </div>
            </div>

            {/* Bar Chart (Weekly Activity) */}
            <div className="bg-white rounded-lg shadow p-6">
              <h2 className="text-lg font-semibold text-gray-800 mb-4">
                Planned Trade Latest
              </h2>
              <div className="h-80">
                <ResponsiveContainer width="100%" height="100%">
                  <BarChart data={weeklyData}>
                    <CartesianGrid strokeDasharray="3 3" />
                    <XAxis dataKey="timestamp" />
                    <YAxis />
                    <Tooltip />
                    <Bar dataKey="ee_fi" fill="#4F46E5" />
                    <Bar dataKey="ee_lv" fill="#00C49F" />
                    <Bar dataKey="ee_ru" fill="#FFBB28" />
                  </BarChart>
                </ResponsiveContainer>
              </div>
            </div>
          </>
        )}
      </div>
    </div>
  );
};

export default Dashboard;
