import React, { useState, useEffect } from "react";
import {
  BarChart,
  Bar,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";
import axios from "axios";

type ChartData = {
  timestamp: string;
  currentPrice?: number;
  latestPrice?: number;
};

const GroupPrice: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [selectedGroup, setSelectedGroup] = useState<string>("EE");
  const [loading, setLoading] = useState<boolean>(false);

  const groups = ["EE", "LT", "LV", "FI"];

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        // Fetch current price data
        const currentResponse = await axios.get(
          `https://localhost:7224/api/GetGroupPrice/getgroupprice/${selectedGroup}`
        );
        const currentData = currentResponse.data.data.map(
          (item: { price: number; timestamp: number }) => ({
            timestamp: new Date(item.timestamp * 1000).toLocaleDateString(),
            currentPrice: item.price,
          })
        );

        // Fetch latest price data
        const latestResponse = await axios.get(
          `https://localhost:7224/api/GetGroupLatestPrice/getgrouplatestprice/${selectedGroup}`
        );
        const latestData = latestResponse.data.data.map(
          (item: { price: number; timestamp: number }) => ({
            timestamp: new Date(item.timestamp * 1000).toLocaleDateString(),
            latestPrice: item.price,
          })
        );

        // Merge data by timestamp
        const mergedData: { [key: string]: ChartData } = {};
        currentData.forEach(
          (item: { timestamp: string; currentPrice?: number }) => {
            if (!mergedData[item.timestamp]) {
              mergedData[item.timestamp] = { timestamp: item.timestamp };
            }
            mergedData[item.timestamp].currentPrice = item.currentPrice;
          }
        );
        latestData.forEach(
          (item: { timestamp: string; latestPrice?: number }) => {
            if (!mergedData[item.timestamp]) {
              mergedData[item.timestamp] = { timestamp: item.timestamp };
            }
            mergedData[item.timestamp].latestPrice = item.latestPrice;
          }
        );
        setData(Object.values(mergedData));
      } catch (error) {
        console.error("Error fetching group price data", error);
      } finally {
        setLoading(false);
      }
    };

    fetchData();
  }, [selectedGroup]);

  const handleGroupChange = (event: React.ChangeEvent<HTMLSelectElement>) => {
    setSelectedGroup(event.target.value);
  };

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">
        Group Price Data (Stacked Bar Chart)
      </h2>

      {/* Dropdown to select the group */}
      <div className="mb-6 flex justify-center items-center space-x-4">
        <label htmlFor="group" className="text-sm text-gray-700">
          Select Group:
        </label>
        <select
          id="group"
          value={selectedGroup}
          onChange={handleGroupChange}
          className="border border-gray-300 rounded px-4 py-2"
        >
          {groups.map((group) => (
            <option key={group} value={group}>
              {group}
            </option>
          ))}
        </select>
      </div>

      {/* Loading indicator */}
      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <BarChart data={data} barCategoryGap="20%">
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            <Legend />
            <Bar
              dataKey="currentPrice"
              stackId="a"
              fill="#4c51bf"
              name="Current Price"
            />
            <Bar
              dataKey="latestPrice"
              stackId="a"
              fill="#48bb78"
              name="Latest Price"
            />
          </BarChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default GroupPrice;
