import React, { useState, useEffect } from "react";
import { BarChart, Bar, XAxis, YAxis, CartesianGrid, Tooltip, Legend, ResponsiveContainer } from "recharts";
import axios from "axios"; // Import axios

type ChartData = {
  price: number;
  timestamp: string;
};

const GroupPriceCurrent: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [selectedGroup, setSelectedGroup] = useState<string>("EE");
  const [loading, setLoading] = useState<boolean>(false);

  const groups = ["EE", "LT", "LV", "FI"];

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        // Fetch data from the API based on selected group
        const response = await axios.get(`https://localhost:7224/api/GetGroupPrice/getgroupprice/${selectedGroup}`);
        const groupData = response.data.data;

        // Assuming the structure of the data is the same as provided (array with price and timestamp)
        const chartData: ChartData[] = groupData.map((item: { price: number; timestamp: number }) => ({
          price: item.price,
          timestamp: new Date(item.timestamp * 1000).toLocaleDateString(), // Format the timestamp
        }));

        setData(chartData);
      } catch (error) {
        console.error("Error fetching group price data", error);
      } finally {
        setLoading(false);
      }
    };

    fetchData();
  }, [selectedGroup]); // This will re-fetch data when the selected group changes

  const handleGroupChange = (event: React.ChangeEvent<HTMLSelectElement>) => {
    setSelectedGroup(event.target.value);
  };

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">Group Price Current</h2>

      {/* Dropdown to select the group */}
      <div className="mb-6 flex justify-center items-center space-x-4">
        <label htmlFor="group" className="text-sm text-gray-700">Select Group:</label>
        <select
          id="group"
          value={selectedGroup}
          onChange={handleGroupChange}
          className="border border-gray-300 rounded px-4 py-2"
        >
          {groups.map((group) => (
            <option key={group} value={group}>
              {group}
            </option>
          ))}
        </select>
      </div>

      {/* Loading indicator */}
      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <BarChart data={data}>
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            <Legend />
            <Bar dataKey="price" fill="#4c51bf" />
          </BarChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default GroupPriceCurrent;
