import React, { useState, useEffect } from "react";
import {
  LineChart,
  Line,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";
import axios from "axios";

type ChartData = {
  timestamp: string;
  ee_fi: number;
  ee_lv: number;
  ee_ru: number;
};

const PlannedTrade: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [startDate, setStartDate] = useState<string>(
    "2024-05-31T20:59:59.999Z"
  );
  const [endDate, setEndDate] = useState<string>("2024-06-30T20:59:59.999Z");
  const [loading, setLoading] = useState<boolean>(false);

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        const response = await axios.get(
          `https://localhost:7224/api/PlannedTrade/getplannedtrade?start=${encodeURIComponent(
            startDate
          )}&end=${encodeURIComponent(endDate)}`
        );
        const plannedTradeData = response.data.data;

        const chartData: ChartData[] = plannedTradeData.map(
          (item: {
            ee_fi: number;
            ee_lv: number;
            ee_ru: number;
            timestamp: number;
          }) => ({
            ee_fi: item.ee_fi,
            ee_lv: item.ee_lv,
            ee_ru: item.ee_ru,
            timestamp: new Date(item.timestamp * 1000).toLocaleDateString(),
          })
        );

        setData(chartData);
      } catch (error) {
        console.error("Error fetching planned trade data", error);
      } finally {
        setLoading(false);
      }
    };

    if (startDate && endDate) {
      fetchData();
    }
  }, [startDate, endDate]);

  const handleDateChange = (
    event: React.ChangeEvent<HTMLInputElement>,
    type: "start" | "end"
  ) => {
    const value = event.target.value;
    const formattedDate = new Date(value).toISOString();
    if (type === "start") {
      setStartDate(formattedDate);
    } else {
      setEndDate(formattedDate);
    }
  };

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">
        Planned Trade
      </h2>

      <div className="mb-6 flex justify-between items-center">
        <div className="flex items-center space-x-2">
          <label htmlFor="start-date" className="text-sm text-gray-700">
            Start Date:
          </label>
          <input
            type="datetime-local"
            id="start-date"
            value={startDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "start")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
        <div className="flex items-center space-x-2">
          <label htmlFor="end-date" className="text-sm text-gray-700">
            End Date:
          </label>
          <input
            type="datetime-local"
            id="end-date"
            value={endDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "end")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
      </div>

      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <LineChart data={data}>
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            <Legend />
            <Line
              type="monotone"
              dataKey="ee_fi"
              stroke="#4c51bf"
              name="EE_FI"
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="ee_lv"
              stroke="#e53e3e"
              name="EE_LV"
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="ee_ru"
              stroke="#38b2ac"
              name="EE_RU"
              dot={false}
            />
          </LineChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default PlannedTrade;
