import React, { useState, useEffect } from "react";
import {
  BarChart,
  Bar,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";
import axios from "axios"; // Import axios

type ChartData = {
  timestamp: string;
  ee_fi: number;
  ee_lv: number;
  ee_ru: number;
};

const PlannedTradeLatest: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [startDate, setStartDate] = useState<string>(
    "2024-05-31T20:59:59.999Z"
  );
  const [endDate, setEndDate] = useState<string>("2024-06-30T20:59:59.999Z");
  const [loading, setLoading] = useState<boolean>(false);

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        const response = await axios.get(
          `https://localhost:7224/api/PlannedTradeLatest/getplannedtradelatest?start=${encodeURIComponent(
            startDate
          )}&end=${encodeURIComponent(endDate)}`
        );
        const plannedTradeData = response.data.data;

        // Process the data into the format for the bar chart
        const chartData: ChartData[] = plannedTradeData.map(
          (item: {
            ee_fi: number;
            ee_lv: number;
            ee_ru: number;
            timestamp: number;
          }) => ({
            ee_fi: item.ee_fi,
            ee_lv: item.ee_lv,
            ee_ru: item.ee_ru,
            timestamp: new Date(item.timestamp * 1000).toLocaleDateString(), // Convert timestamp to a readable date
          })
        );

        setData(chartData);
      } catch (error) {
        console.error("Error fetching planned trade latest data", error);
      } finally {
        setLoading(false);
      }
    };

    // Fetch data when both startDate and endDate are selected
    if (startDate && endDate) {
      fetchData();
    }
  }, [startDate, endDate]); // This will run whenever the start or end date changes

  // Handle date change events
  const handleDateChange = (
    event: React.ChangeEvent<HTMLInputElement>,
    type: "start" | "end"
  ) => {
    const value = event.target.value;
    const formattedDate = new Date(value).toISOString(); // Format date to ISO string
    if (type === "start") {
      setStartDate(formattedDate);
    } else {
      setEndDate(formattedDate);
    }
  };

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">
        Planned Trade Latest
      </h2>

      {/* Date range picker */}
      <div className="mb-6 flex justify-between items-center">
        <div className="flex items-center space-x-2">
          <label htmlFor="start-date" className="text-sm text-gray-700">
            Start Date:
          </label>
          <input
            type="datetime-local"
            id="start-date"
            value={startDate.slice(0, 19)} // Remove milliseconds for the input value
            onChange={(e) => handleDateChange(e, "start")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
        <div className="flex items-center space-x-2">
          <label htmlFor="end-date" className="text-sm text-gray-700">
            End Date:
          </label>
          <input
            type="datetime-local"
            id="end-date"
            value={endDate.slice(0, 19)} // Remove milliseconds for the input value
            onChange={(e) => handleDateChange(e, "end")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
      </div>

      {/* Loading indicator */}
      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <BarChart data={data}>
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            <Legend />
            {/* Bar for "ee_fi" */}
            <Bar dataKey="ee_fi" fill="#4c51bf" name="EE_FI" />
            {/* Bar for "ee_lv" */}
            <Bar dataKey="ee_lv" fill="#e53e3e" name="EE_LV" />
            {/* Bar for "ee_ru" */}
            <Bar dataKey="ee_ru" fill="#38b2ac" name="EE_RU" />
          </BarChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default PlannedTradeLatest;
