import React, { useState, useEffect } from "react";
import {
  LineChart,
  Line,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";
import axios from "axios";

type ChartData = {
  region: string;
  price: number;
  timestamp: string;
};

const Price: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [startDate, setStartDate] = useState<string>(
    "2024-05-31T20:59:59.999Z"
  );
  const [endDate, setEndDate] = useState<string>("2024-06-30T20:59:59.999Z");
  const [loading, setLoading] = useState<boolean>(false);
  const [visibleRegions, setVisibleRegions] = useState<string[]>([
    "ee",
    "fi",
    "lt",
    "lv",
  ]);

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        const response = await axios.get(
          `https://localhost:7224/api/Price/getprice?start=${encodeURIComponent(
            startDate
          )}&end=${encodeURIComponent(endDate)}`
        );

        const regionData = response.data.data;
        const chartData: ChartData[] = [];
        Object.keys(regionData).forEach((region) => {
          regionData[region].forEach(
            (item: { price: number; timestamp: number }) => {
              chartData.push({
                region,
                price: item.price,
                timestamp: new Date(item.timestamp * 1000).toLocaleDateString(),
              });
            }
          );
        });

        setData(chartData);
      } catch (error) {
        console.error("Error fetching price data", error);
      } finally {
        setLoading(false);
      }
    };

    if (startDate && endDate) {
      fetchData();
    }
  }, [startDate, endDate]);

  const handleDateChange = (
    event: React.ChangeEvent<HTMLInputElement>,
    type: "start" | "end"
  ) => {
    const value = event.target.value;
    const formattedDate = new Date(value).toISOString();
    if (type === "start") {
      setStartDate(formattedDate);
    } else {
      setEndDate(formattedDate);
    }
  };

  const toggleRegion = (region: string) => {
    setVisibleRegions((prev) =>
      prev.includes(region)
        ? prev.filter((r) => r !== region)
        : [...prev, region]
    );
  };

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">
        Price Chart
      </h2>

      <div className="mb-6 flex justify-between items-center">
        <div className="flex items-center space-x-2">
          <label htmlFor="start-date" className="text-sm text-gray-700">
            Start Date:
          </label>
          <input
            type="datetime-local"
            id="start-date"
            value={startDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "start")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
        <div className="flex items-center space-x-2">
          <label htmlFor="end-date" className="text-sm text-gray-700">
            End Date:
          </label>
          <input
            type="datetime-local"
            id="end-date"
            value={endDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "end")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
      </div>

      <div className="mb-4 flex space-x-4">
        {["ee", "fi", "lt", "lv"].map((region) => (
          <button
            key={region}
            onClick={() => toggleRegion(region)}
            className={`px-4 py-2 rounded ${
              visibleRegions.includes(region)
                ? "bg-indigo-600 text-white"
                : "bg-gray-200 text-gray-800"
            }`}
          >
            {region.toUpperCase()}
          </button>
        ))}
      </div>

      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <LineChart
            data={data.filter((item) => visibleRegions.includes(item.region))}
          >
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            <Legend />
            {["ee", "fi", "lt", "lv"].map(
              (region) =>
                visibleRegions.includes(region) && (
                  <Line
                    key={region}
                    type="monotone"
                    dataKey="price"
                    stroke={
                      {
                        ee: "#ff6384",
                        fi: "#36a2eb",
                        lt: "#ffce56",
                        lv: "#4bc0c0",
                      }[region]
                    }
                    name={region.toUpperCase()}
                  />
                )
            )}
          </LineChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default Price;
