import React, { useState, useEffect } from "react";
import {
  LineChart,
  Line,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";
import axios from "axios";

type ChartData = {
  timestamp: string;
  in_atc: number;
  in_ntc: number;
  out_atc: number;
  out_ntc: number;
};

const TransmissionBorderCapacity: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [startDate, setStartDate] = useState<string>(
    "2024-05-31T20:59:59.999Z"
  );
  const [endDate, setEndDate] = useState<string>("2024-06-30T20:59:59.999Z");
  const [loading, setLoading] = useState<boolean>(false);

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        const response = await axios.get(
          `https://localhost:7224/api/TransmissionBorderCapacity/getcapacity?start=${encodeURIComponent(
            startDate
          )}&end=${encodeURIComponent(endDate)}`
        );
        const regionData = response.data.data;

        // Process the data into the format for the line chart
        const chartData: ChartData[] = [];

        // Process each region (FI, LV, RU)
        Object.keys(regionData).forEach((region) => {
          regionData[region].forEach(
            (item: {
              in_atc: number;
              in_ntc: number;
              out_atc: number;
              out_ntc: number;
              timestamp: number;
            }) => {
              chartData.push({
                in_atc: item.in_atc,
                in_ntc: item.in_ntc,
                out_atc: item.out_atc,
                out_ntc: item.out_ntc,
                timestamp: new Date(item.timestamp * 1000).toLocaleDateString(),
              });
            }
          );
        });

        setData(chartData);
      } catch (error) {
        console.error(
          "Error fetching transmission border capacity data",
          error
        );
      } finally {
        setLoading(false);
      }
    };

    if (startDate && endDate) {
      fetchData();
    }
  }, [startDate, endDate]);

  const handleDateChange = (
    event: React.ChangeEvent<HTMLInputElement>,
    type: "start" | "end"
  ) => {
    const value = event.target.value;
    const formattedDate = new Date(value).toISOString();
    if (type === "start") {
      setStartDate(formattedDate);
    } else {
      setEndDate(formattedDate);
    }
  };

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">
        Transmission Border Capacity
      </h2>

      <div className="mb-6 flex justify-between items-center">
        <div className="flex items-center space-x-2">
          <label htmlFor="start-date" className="text-sm text-gray-700">
            Start Date:
          </label>
          <input
            type="datetime-local"
            id="start-date"
            value={startDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "start")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
        <div className="flex items-center space-x-2">
          <label htmlFor="end-date" className="text-sm text-gray-700">
            End Date:
          </label>
          <input
            type="datetime-local"
            id="end-date"
            value={endDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "end")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
      </div>

      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <LineChart data={data}>
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            <Legend />
            <Line
              type="monotone"
              dataKey="in_atc"
              stroke="#4c51bf"
              name="In ATC"
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="in_ntc"
              stroke="#38b2ac"
              name="In NTC"
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="out_atc"
              stroke="#e53e3e"
              name="Out ATC"
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="out_ntc"
              stroke="#ed8936"
              name="Out NTC"
              dot={false}
            />
          </LineChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default TransmissionBorderCapacity;
