import React, { useState, useEffect } from "react";
import {
  LineChart,
  Line,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";
import axios from "axios";

type ChartData = {
  timestamp: string;
  estlink_1: number;
  estlink_2: number;
  finland: number;
  latvia: number;
  russia_narva: number;
  russia_pihkva: number;
};

// Define a custom type for the legend click payload
interface LegendClickPayload {
  value: string; // The name shown in the legend (e.g., "Estlink 1")
}

const TransmissionHourly: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [startDate, setStartDate] = useState<string>(
    "2024-05-31T20:59:59.999Z"
  );
  const [endDate, setEndDate] = useState<string>("2024-06-30T20:59:59.999Z");
  const [loading, setLoading] = useState<boolean>(false);

  // Keep track of which lines are hidden
  const [hiddenLines, setHiddenLines] = useState<string[]>([]);

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        const response = await axios.get(
          `https://localhost:7224/api/TransmissionHourly/gettransmissionhourly?start=${encodeURIComponent(
            startDate
          )}&end=${encodeURIComponent(endDate)}`
        );
        const transmissionData = response.data.data;

        if (transmissionData && transmissionData.length > 0) {
          const chartData: ChartData[] = transmissionData.map(
            (item: {
              estlink_1: number;
              estlink_2: number;
              finland: number;
              latvia: number;
              russia_narva: number;
              russia_pihkva: number;
              timestamp: number;
            }) => ({
              estlink_1: item.estlink_1,
              estlink_2: item.estlink_2,
              finland: item.finland,
              latvia: item.latvia,
              russia_narva: item.russia_narva,
              russia_pihkva: item.russia_pihkva,
              timestamp: new Date(item.timestamp * 1000).toLocaleDateString(),
            })
          );

          setData(chartData);
        } else {
          console.error("No transmission data available");
        }
      } catch (error) {
        console.error("Error fetching transmission hourly data", error);
      } finally {
        setLoading(false);
      }
    };

    if (startDate && endDate) {
      fetchData();
    }
  }, [startDate, endDate]);

  // Update your handleLegendClick to use your custom LegendClickPayload type
  const handleLegendClick = (o: LegendClickPayload) => {
    const { value } = o;
    if (hiddenLines.includes(value)) {
      setHiddenLines(hiddenLines.filter((line) => line !== value));
    } else {
      setHiddenLines([...hiddenLines, value]);
    }
  };

  // Date change handler
  const handleDateChange = (
    event: React.ChangeEvent<HTMLInputElement>,
    type: "start" | "end"
  ) => {
    const value = event.target.value;
    const formattedDate = new Date(value).toISOString();
    if (type === "start") {
      setStartDate(formattedDate);
    } else {
      setEndDate(formattedDate);
    }
  };

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">
        Transmission Hourly
      </h2>

      {/* Date range picker */}
      <div className="mb-6 flex justify-between items-center">
        <div className="flex items-center space-x-2">
          <label htmlFor="start-date" className="text-sm text-gray-700">
            Start Date:
          </label>
          <input
            type="datetime-local"
            id="start-date"
            value={startDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "start")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
        <div className="flex items-center space-x-2">
          <label htmlFor="end-date" className="text-sm text-gray-700">
            End Date:
          </label>
          <input
            type="datetime-local"
            id="end-date"
            value={endDate.slice(0, 19)}
            onChange={(e) => handleDateChange(e, "end")}
            className="border border-gray-300 rounded px-3 py-2"
          />
        </div>
      </div>

      {/* Loading indicator */}
      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <LineChart data={data}>
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            {/* Pass our custom handler to the Legend */}
            <Legend onClick={handleLegendClick} />

            {/* Use `hide` prop to conditionally show/hide lines */}
            <Line
              type="bump"
              dataKey="estlink_1"
              stroke="#4c51bf"
              name="Estlink 1"
              hide={hiddenLines.includes("Estlink 1")}
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="estlink_2"
              stroke="#38b2ac"
              name="Estlink 2"
              hide={hiddenLines.includes("Estlink 2")}
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="finland"
              stroke="#e53e3e"
              name="Finland"
              hide={hiddenLines.includes("Finland")}
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="latvia"
              stroke="#ed8936"
              name="Latvia"
              hide={hiddenLines.includes("Latvia")}
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="russia_narva"
              stroke="#f6ad55"
              name="Russia Narva"
              hide={hiddenLines.includes("Russia Narva")}
              dot={false}
            />
            <Line
              type="monotone"
              dataKey="russia_pihkva"
              stroke="#fbd38d"
              name="Russia Pihkva"
              hide={hiddenLines.includes("Russia Pihkva")}
              dot={false}
            />
          </LineChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default TransmissionHourly;
