import React, { useState, useEffect } from "react";
import {
  BarChart,
  Bar,
  XAxis,
  YAxis,
  CartesianGrid,
  Tooltip,
  Legend,
  ResponsiveContainer,
} from "recharts";
import axios from "axios"; // Import axios

type ChartData = {
  timestamp: string;
  finland: number;
  latvia: number;
  estlink_1: number;
  estlink_2: number;
  russia_narva: number;
  russia_pihkva: number;
};

const TransmissionLatest: React.FC = () => {
  const [data, setData] = useState<ChartData[]>([]);
  const [loading, setLoading] = useState<boolean>(false);

  useEffect(() => {
    const fetchData = async () => {
      setLoading(true);
      try {
        const response = await axios.get(
          `https://localhost:7224/api/TransmissionLatest/transmissionlatest`
        );
        const transmissionData = response.data.data;

        if (transmissionData && transmissionData.length > 0) {
          // Process the data into the format for the bar chart
          const chartData: ChartData[] = transmissionData.map(
            (item: {
              finland: number;
              latvia: number;
              estlink_1: number;
              estlink_2: number;
              russia_narva: number;
              russia_pihkva: number;
              timestamp: number;
            }) => ({
              finland: item.finland,
              latvia: item.latvia,
              estlink_1: item.estlink_1,
              estlink_2: item.estlink_2,
              russia_narva: item.russia_narva,
              russia_pihkva: item.russia_pihkva,
              timestamp: new Date(item.timestamp * 1000).toLocaleDateString(), // Convert timestamp to a readable date
            })
          );

          setData(chartData);
        } else {
          console.error("No transmission data available");
        }
      } catch (error) {
        console.error("Error fetching transmission cross-border data", error);
      } finally {
        setLoading(false);
      }
    };

    fetchData();
  }, []); // Fetch data once when the component mounts

  return (
    <div className="p-6 bg-gray-50 rounded-lg shadow-lg">
      <h2 className="text-xl font-semibold mb-6 text-center text-gray-800">
        Transmission Cross Border Latest
      </h2>

      {/* Loading indicator */}
      {loading ? (
        <div className="text-center text-gray-600">Loading data...</div>
      ) : (
        <ResponsiveContainer width="100%" height={400}>
          <BarChart data={data}>
            <CartesianGrid strokeDasharray="3 3" />
            <XAxis dataKey="timestamp" />
            <YAxis />
            <Tooltip />
            <Legend />
            {/* Bar for "estlink_1" */}
            <Bar dataKey="estlink_1" fill="#4c51bf" name="Estlink 1" />
            {/* Bar for "estlink_2" */}
            <Bar dataKey="estlink_2" fill="#38b2ac" name="Estlink 2" />
            {/* Bar for "finland" */}
            <Bar dataKey="finland" fill="#e53e3e" name="Finland" />
            {/* Bar for "latvia" */}
            <Bar dataKey="latvia" fill="#ed8936" name="Latvia" />
            {/* Bar for "russia_narva" */}
            <Bar dataKey="russia_narva" fill="#f6ad55" name="Russia Narva" />
            {/* Bar for "russia_pihkva" */}
            <Bar dataKey="russia_pihkva" fill="#fbd38d" name="Russia Pihkva" />
          </BarChart>
        </ResponsiveContainer>
      )}
    </div>
  );
};

export default TransmissionLatest;
